import 'dart:io';
import 'dart:typed_data';

import 'package:flutter_curiosity/flutter_curiosity.dart';
import 'package:flutter_curiosity/src/tools/internal.dart';

///安装apk  仅支持android
///success  安装成功
///cancel  取消安装
///not permissions  没有打开安装权限
Future<String> installApp(String apkPath) async {
  if (!isAndroid) return null;
  return await curiosityChannel
      .invokeMethod('installApp', <String, String>{'apkPath': apkPath});
}

///去应用市场 android 安装多个应用商店时会弹窗选择，ios app store
///android => packageName
///ios => app id
///The android platform "marketPackageName" cannot be null
Future<void> goToMarket<T>(
    {String packageName, String marketPackageName, String appID}) async {
  if (!InternalTools.supportPlatform()) {
    if (isIOS) {
      assert(appID != null);
      await curiosityChannel
          .invokeMethod<T>('goToMarket', <String, String>{'appId': appID});
    }
    if (isAndroid) {
      assert(packageName != null && marketPackageName != null);
      if (marketPackageName == null)
        await curiosityChannel.invokeMethod<T>('goToMarket', <String, String>{
          'packageName': packageName,
          'marketPackageName': marketPackageName
        });
    }
  }
}

///是否安装某个app  仅支持android
Future<bool> isInstallApp(String packageName) async {
  if (isAndroid) return false;
  return await curiosityChannel.invokeMethod(
      'isInstallApp', <String, String>{'packageName': packageName});
}

///退出app
Future<void> get exitApp async {
  if (!InternalTools.supportPlatform())
    await curiosityChannel.invokeMethod<dynamic>('exitApp');
}

///获取文件夹或文件大小
Future<String> getFilePathSize(String path) async {
  if (InternalTools.supportPlatform()) return null;
  return await curiosityChannel
      .invokeMethod('getFilePathSize', <String, String>{'filePath': path});
}

///拨打电话
///directDial true 直接拨打电话 false 跳转到拨号页面并输入手机号
///directDial 为 true 需要 自行申请动态申请权限
Future<void> systemCallPhone<T>(String phoneNumber,
    {bool directDial = false}) async {
  if (InternalTools.supportPlatform()) return;
  await curiosityChannel.invokeMethod<T>('callPhone',
      <String, dynamic>{'phoneNumber': phoneNumber, 'directDial': directDial});
}

///系统分享
Future<String> systemShare(
    {String title = 'Share',
    String content,
    List<String> imagesPath,
    ShareType shareType}) async {
  if (InternalTools.supportPlatform())
    return 'not support ${Platform.operatingSystem}';
  if (shareType == null) return 'The shareType cannot be empty';

  if (shareType == ShareType.images) {
    if (imagesPath == null || imagesPath.isEmpty)
      return 'The shareType cannot be empty';
  }
  if (content == null && imagesPath == null)
    return 'A share parameter must be passed content or imagesPath';
  if (content != null && imagesPath != null)
    return 'Only one parameter can be passed';
  return await curiosityChannel.invokeMethod('systemShare', <String, dynamic>{
    'title': title,
    'content': content,
    'type': shareType.toString().split('.')[1],
    'imagesPath': imagesPath
  });
}

///判断GPS是否开启，GPS或者AGPS开启一个就认为是开启的
Future<bool> get getGPSStatus async {
  if (InternalTools.supportPlatform()) return null;
  return await curiosityChannel.invokeMethod('getGPSStatus');
}

///跳转到GPS定位权限设置页面
Future<bool> get jumpGPSSetting async {
  if (InternalTools.supportPlatform()) return null;
  if (isIOS) return await jumpAppSetting;
  if (isAndroid) return await curiosityChannel.invokeMethod('jumpGPSSetting');
  return null;
}

///跳转到App权限设置页面
Future<bool> get jumpAppSetting async {
  if (InternalTools.supportPlatform()) return null;
  return await curiosityChannel.invokeMethod('jumpAppSetting');
}

///GalleryTools
///
///  openPicker() async {
///    PicturePickerOptions pickerOptions = PicturePickerOptions();
///    pickerOptions.selectValueType = 0;
///    pickerOptions.previewVideo = true;
///    var list = await PicturePicker.openPicker(pickerOptions);
///    print(list);
///  }
Future<List<AssetMedia>> openImagePicker(
    [PicturePickerOptions selectOptions]) async {
  if (InternalTools.supportPlatform()) return null;
  selectOptions ??= PicturePickerOptions();
  if (selectOptions.maxSelectNum < 1) selectOptions.maxSelectNum = 1;
  final List<dynamic> result = await curiosityChannel.invokeMethod(
      'openImagePicker', selectOptions.toJson());
  if (result is List) {
    final List<AssetMedia> list = <AssetMedia>[];
    for (final dynamic data in result) {
      final Map<dynamic, dynamic> imageData = data as Map<dynamic, dynamic>;
      list.add(AssetMedia.fromJson(imageData));
    }
    log(list);
    return list;
  } else {
    return null;
  }
}

/// [selectValueType] 0:全部类型，1:图片，2:视频
Future<String> deleteCacheDirFile({int selectValueType = 0}) async {
  if (InternalTools.supportPlatform()) return null;
  return curiosityChannel.invokeMethod(
      'deleteCacheDirFile', <String, int>{'selectValueType': selectValueType});
}

///打开系统相册
///返回文件路径
///ios info.plist add
///       <key>NSPhotoLibraryUsageDescription</key>
///       <string>是否允许Curiosity访问你的相册？</string>
///ios path 包含 file:///
Future<String> get openSystemGallery async {
  if (InternalTools.supportPlatform()) return null;
  return await curiosityChannel.invokeMethod('openSystemGallery');
}

///打开系统相机
///返回文件路径
///Android AndroidManifest.xml 添加以下内容
///<application
///          ...>
///   <provider
///            android:name="androidx.core.content.FileProvider"
///            android:authorities="${applicationId}.fileprovider"
///            android:exported="false"
///            android:grantUriPermissions="true">
///            <meta-data
///                android:name="android.support.FILE_PROVIDER_PATHS"
///                android:resource="@xml/file_paths" />
///   </provider>
///</application>
///ios info.plist add
///     <key>NSCameraUsageDescription</key>
///       <string>是否允许APP使用你的相机？</string>
///      <key>NSPhotoLibraryUsageDescription</key>
///       <string>是否允许APP访问你的相册？</string>
///ios path 包含 file:///
Future<String> openSystemCamera({String savePath}) async {
  ///savePath => android 图片临时储存位置 (仅支持android)
  ///alertNativeTips => ios 是否弹出用户未允许访问相机权限提示 (仅支持ios)
  if (InternalTools.supportPlatform()) return null;
  String path = await curiosityChannel
      .invokeMethod('openSystemCamera', <String, String>{'path': savePath});
  if (savePath != null) path = savePath;
  return path;
}

/// save image to Gallery
/// imageBytes can't null
Future<String> saveImageToGallery(Uint8List imageBytes,
    {int quality = 100, String name}) async {
  assert(imageBytes != null);
  final String result = await curiosityChannel.invokeMethod(
      'saveImageToGallery', <String, dynamic>{
    'imageBytes': imageBytes,
    'quality': quality,
    'name': name
  });
  return result;
}

/// Save the PNG，JPG，JPEG image or video located at [file] to the local device media gallery.
Future<String> saveFileToGallery(String file) async {
  assert(file != null);
  final String result =
      await curiosityChannel.invokeMethod('saveFileToGallery', file);
  return result;
}

///AppInfo

///get all info
Future<AppInfoModel> get getPackageInfo async {
  if (InternalTools.supportPlatform()) return null;
  final Map<String, dynamic> map =
      await curiosityChannel.invokeMapMethod<String, dynamic>('getAppInfo');
  return AppInfoModel.fromJson(map);
}

///android versionCode  ios version
Future<int> get getVersionCode async {
  final AppInfoModel appInfoModel = await getPackageInfo;
  return appInfoModel.versionCode;
}

///app name
Future<String> get getAppName async {
  if (isIOS || isAndroid) {
    final AppInfoModel appInfoModel = await getPackageInfo;
    return appInfoModel.appName;
  }
  return null;
}

///package name
Future<String> get getPackageName async {
  final AppInfoModel appInfoModel = await getPackageInfo;
  return appInfoModel.packageName;
}

///android versionName  ios buildName
Future<String> get getVersionName async {
  final AppInfoModel appInfoModel = await getPackageInfo;
  return appInfoModel.versionName;
}

///root directory
Future<String> get getRootDirectory async {
  final AppInfoModel appInfoModel = await getPackageInfo;
  if (isAndroid) return appInfoModel.externalStorageDirectory;
  if (isIOS || isMacOS) return appInfoModel.homeDirectory;
  return '';
}
